package com.Conquistadores.App;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Timer;
import java.util.TimerTask;

import org.json.JSONObject;

import com.Conquistadores.App.R;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

/**
 * Servicio que se ejecuta peridicamente para informar al usuario si ha recibido mensajes.
 * En el caso de recibirlos, se genera una notificacin en la barra de estado con el fin
 * de informar al jugador.
 * Hasta que el usuario no vea la notificacin generada, el servicio se lo recordar.
 * 
 * @author Fco. Javier Quero Castrillo <xavifuenla@gmail.com>
 * 
 */
public class ServicioMensajeria extends Service{
	
	static private String TAG = "ServicioMensajeria";
	
	private static final int RETARDO = 5 * 1000;
	private static final int PERIODO = 45 * 1000;
	private static final int RESULTADO_ERROR = -1;
	private static final int RESULTADO_OK = 0;
	private static final int MENSAJE_ID = 1;

	private int mensajeServidorId;
	private static int mensajeAplicacionId;
	private NotificationManager mNotificationManager;
	private String url;
	private Timer timer;
	
	public static void setMensajeAplicacionId(int mensajeAplicacionId) {
		ServicioMensajeria.mensajeAplicacionId = mensajeAplicacionId;
	}

	@Override
	public void onCreate(){
		super.onCreate();
		// Iniciamos el servicio
		mensajeServidorId = 0;
		mensajeAplicacionId = 0;
		this.iniciarServicio();
	}

	@Override
	public void onDestroy(){
		super.onDestroy();
		// Detenemos el servicio
		this.finalizarServicio();
	}
	
	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}

	private void iniciarServicio(){
		String ns = Context.NOTIFICATION_SERVICE;
		mNotificationManager = (NotificationManager) getSystemService(ns);
		
		String urlServer = "http://"+getString(R.string.urlServer).replaceFirst("/social/", "/");		
		int partidaId = PartidaListar.getPartida().getIdent();
		int equipoId = RecursoListar.getEquipo().getIdent();
		url = String.format("%1$sconquistadores/partida/%2$s/equipo/%3$s/servicio/mensajeria/?format=json",
				urlServer,partidaId,equipoId);
		
		TimerTask timerTask = new TimerTask(){
			public void run(){
				obtenerHistMensajes();
			}
		};
		
		// Creamos el timer
		timer = new Timer();
		timer.scheduleAtFixedRate(timerTask, RETARDO, PERIODO);
	}

	private void finalizarServicio(){
		// Detenemos el timer
		if (timer != null)
			this.timer.cancel();
		mNotificationManager.cancelAll();
	}
	
    private Handler handler = new Handler(){
    	public void handleMessage(Message msg){
    		int valor = msg.what;
    		switch(valor){
    		case RESULTADO_ERROR:   		
    			Context context = getApplicationContext();
    			CharSequence text = getString(R.string.toastFalloServMsg);
    			Toast toast = Toast.makeText(context, text, Toast.LENGTH_SHORT);
    			toast.setGravity(Gravity.CENTER, 0, 0);
    			toast.show();
    			break;
    		case RESULTADO_OK:   			
    			if(mensajeServidorId > mensajeAplicacionId){
    				int icon = R.drawable.ic_stat_notify_msg_recb;
    				CharSequence tickerText = getString(R.string.tickerNotificationMsg);
    				CharSequence contentTitle = getString(R.string.tituloNotificationMsg);
    				CharSequence contentText = getString(R.string.textoNotificationMsg);
    				showBarNotification(icon, tickerText, contentTitle, contentText);	   				
    			}   				
    			break;
    		};
    	}
    };
	
	private void obtenerHistMensajes(){
		int resultado;

		try{	
			String str = Utilidades.obtenerDatos(url);
			resultado = procesarRespuestaMensajeRecibidos(str);
			handler.sendEmptyMessage(resultado);

		}catch(URISyntaxException urisex){
			handler.sendEmptyMessage(RESULTADO_ERROR);
	    }catch (IOException e) {
	    	handler.sendEmptyMessage(RESULTADO_ERROR);
	    }
	}
    
    private int procesarRespuestaMensajeRecibidos (String str) {
    	int code;
    	String info;
    	
        try {
        	JSONObject json;
        	json = new JSONObject(str.toString());

			if (json.has("codigo"))
				code = json.getInt("codigo");
			else
				code = -1;

			if (json.has("info"))
				info = json.getString("info");
			else
				info = null;
			
			if (code != 200)
				return RESULTADO_ERROR;
			
			if (json.has("ident"))
				mensajeServidorId = json.getInt("ident");
			else
				mensajeServidorId = -1;
		} catch (Exception e) {	
			code = 500;
			info = "Error en el parsed json: " + e.getMessage();
			Log.e(TAG, info);
			return RESULTADO_ERROR;
		}
		return RESULTADO_OK;
	}
    
	private void showBarNotification(int icon, CharSequence tickerText, CharSequence contentTitle, CharSequence contentText){	
		long when = System.currentTimeMillis();
		Notification notification = new Notification(icon, tickerText, when);
		
		Intent intent = new Intent(ServicioMensajeria.this, MensajeListar.class);
		Bundle bundle = new Bundle();
		bundle.putString("tipo_mensaje", "r");
		intent.putExtras(bundle);
		PendingIntent contentIntent = PendingIntent.getActivity(ServicioMensajeria.this, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
		
		notification.setLatestEventInfo(ServicioMensajeria.this, contentTitle, contentText, contentIntent);
		  				
		notification.defaults |= Notification.DEFAULT_VIBRATE;
		
		notification.ledARGB = 0xff00ff00;
		notification.ledOnMS = 500;
		notification.ledOffMS = 1000;
		notification.flags |= Notification.FLAG_SHOW_LIGHTS;
		
		notification.flags |= Notification.FLAG_AUTO_CANCEL;
		
		mNotificationManager.notify(MENSAJE_ID, notification); 
	}
}
